==> Perinci-CmdLine-Any-Bundled-0.01.tar.gz: Perinci-CmdLine-Any-Bundled-0.01/lib/Data/Sah/Type/hash.pm <==
package Data::Sah::Type::hash;

our $DATE = '2015-05-13'; # DATE
our $VERSION = '0.56'; # VERSION

use Data::Sah::Util::Role 'has_clause', 'has_clause_alias';
use Role::Tiny;
use Role::Tiny::With;

with 'Data::Sah::Type::BaseType';
with 'Data::Sah::Type::Comparable';
with 'Data::Sah::Type::HasElems';

has_clause_alias each_elem => 'of';

has_clause "keys",
    tags       => ['constraint'],
    arg        => ['hash*' => {values => 'schema*'}],
    allow_expr => 0,
    attrs      => {
        restrict => {
            arg        => [bool => default=>1],
            allow_expr => 0, # TODO
        },
        create_default => {
            arg        => [bool => default=>1],
            allow_expr => 0, # TODO
        },
    },
    ;
has_clause "re_keys",
    prio       => 51,
    tags       => ['constraint'],
    arg        => ['hash*' => {keys => 're*', values => 'schema*'}],
    allow_expr => 0,
    attrs      => {
        restrict => {
            arg        => [bool => default=>1],
            allow_expr => 0, # TODO
        },
    },
    ;
has_clause "req_keys",
    tags       => ['constraint'],
    arg        => ['array*'],
    allow_expr => 1,
    ;
has_clause "allowed_keys",
    tags       => ['constraint'],
    arg        => ['array*'],
    allow_expr => 1,
    ;
has_clause "allowed_keys_re",
    prio       => 51,
    tags       => ['constraint'],
    arg        => 're*',
    allow_expr => 1,
    ;
has_clause "forbidden_keys",
    tags       => ['constraint'],
    arg        => ['array*'],
    allow_expr => 1,
    ;
has_clause "forbidden_keys_re",
    prio       => 51,
    tags       => ['constraint'],
    arg        => 're*',
    allow_expr => 1,
    ;
has_clause_alias each_index => 'each_key';
has_clause_alias each_elem => 'each_value';
has_clause_alias check_each_index => 'check_each_key';
has_clause_alias check_each_elem => 'check_each_value';

has_clause "choose_one_key",
    prio       => 50,
    tags       => ['constraint'],
    arg        => ['array*', {of=>'str*', min_len=>1}],
    allow_expr => 0, # for now
    ;
has_clause_alias choose_one_key => 'choose_one';
has_clause "choose_all_keys",
    prio       => 50,
    tags       => ['constraint'],
    arg        => ['array*', {of=>'str*', min_len=>1}],
    allow_expr => 0, # for now
    ;
has_clause_alias choose_all_keys => 'choose_all';
has_clause "req_one_key",
    prio       => 50,
    tags       => ['constraint'],
    arg        => ['array*', {of=>'str*', min_len=>1}],
    allow_expr => 0, # for now
    ;
has_clause_alias req_one_key => 'req_one';
has_clause_alias req_keys => 'req_all_keys';
has_clause_alias req_keys => 'req_all';

# for now we only support the first argument as str, not array[str]
#my $dep_arg = ['array*', {elems=>[ ['any*', of=>['str*', ['array*',{of=>'str*'}]]], ['array*',of=>'str*'] ]}];
my $dep_arg = ['array*', {elems=>[ 'str*', ['array*',of=>'str*'] ]}];

has_clause "dep_any",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;
has_clause "dep_all",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;
has_clause "req_dep_any",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;
has_clause "req_dep_all",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;

# prop_alias indices => 'keys'

# prop_alias elems => 'values'

1;
# ABSTRACT: hash type

__END__

=pod

=encoding UTF-8

=head1 NAME

Data::Sah::Type::hash - hash type

=head1 VERSION

This document describes version 0.56 of Data::Sah::Type::hash (from Perl distribution Perinci-CmdLine-Any-Bundled version 0.01), released on 2015-05-13.

=for Pod::Coverage ^(clause_.+|clausemeta_.+)$

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Data-Sah>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Data-Sah>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Data-Sah>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

==> Perinci-CmdLine-Any-Bundled-0.02.tar.gz: Perinci-CmdLine-Any-Bundled-0.02/lib/Data/Sah/Type/hash.pm <==
package Data::Sah::Type::hash;

our $DATE = '2015-05-13'; # DATE
our $VERSION = '0.56'; # VERSION

use Data::Sah::Util::Role 'has_clause', 'has_clause_alias';
use Role::Tiny;
use Role::Tiny::With;

with 'Data::Sah::Type::BaseType';
with 'Data::Sah::Type::Comparable';
with 'Data::Sah::Type::HasElems';

has_clause_alias each_elem => 'of';

has_clause "keys",
    tags       => ['constraint'],
    arg        => ['hash*' => {values => 'schema*'}],
    allow_expr => 0,
    attrs      => {
        restrict => {
            arg        => [bool => default=>1],
            allow_expr => 0, # TODO
        },
        create_default => {
            arg        => [bool => default=>1],
            allow_expr => 0, # TODO
        },
    },
    ;
has_clause "re_keys",
    prio       => 51,
    tags       => ['constraint'],
    arg        => ['hash*' => {keys => 're*', values => 'schema*'}],
    allow_expr => 0,
    attrs      => {
        restrict => {
            arg        => [bool => default=>1],
            allow_expr => 0, # TODO
        },
    },
    ;
has_clause "req_keys",
    tags       => ['constraint'],
    arg        => ['array*'],
    allow_expr => 1,
    ;
has_clause "allowed_keys",
    tags       => ['constraint'],
    arg        => ['array*'],
    allow_expr => 1,
    ;
has_clause "allowed_keys_re",
    prio       => 51,
    tags       => ['constraint'],
    arg        => 're*',
    allow_expr => 1,
    ;
has_clause "forbidden_keys",
    tags       => ['constraint'],
    arg        => ['array*'],
    allow_expr => 1,
    ;
has_clause "forbidden_keys_re",
    prio       => 51,
    tags       => ['constraint'],
    arg        => 're*',
    allow_expr => 1,
    ;
has_clause_alias each_index => 'each_key';
has_clause_alias each_elem => 'each_value';
has_clause_alias check_each_index => 'check_each_key';
has_clause_alias check_each_elem => 'check_each_value';

has_clause "choose_one_key",
    prio       => 50,
    tags       => ['constraint'],
    arg        => ['array*', {of=>'str*', min_len=>1}],
    allow_expr => 0, # for now
    ;
has_clause_alias choose_one_key => 'choose_one';
has_clause "choose_all_keys",
    prio       => 50,
    tags       => ['constraint'],
    arg        => ['array*', {of=>'str*', min_len=>1}],
    allow_expr => 0, # for now
    ;
has_clause_alias choose_all_keys => 'choose_all';
has_clause "req_one_key",
    prio       => 50,
    tags       => ['constraint'],
    arg        => ['array*', {of=>'str*', min_len=>1}],
    allow_expr => 0, # for now
    ;
has_clause_alias req_one_key => 'req_one';
has_clause_alias req_keys => 'req_all_keys';
has_clause_alias req_keys => 'req_all';

# for now we only support the first argument as str, not array[str]
#my $dep_arg = ['array*', {elems=>[ ['any*', of=>['str*', ['array*',{of=>'str*'}]]], ['array*',of=>'str*'] ]}];
my $dep_arg = ['array*', {elems=>[ 'str*', ['array*',of=>'str*'] ]}];

has_clause "dep_any",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;
has_clause "dep_all",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;
has_clause "req_dep_any",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;
has_clause "req_dep_all",
    prio       => 50,
    tags       => ['constraint'],
    arg        => $dep_arg,
    allow_expr => 0, # for now
    ;

# prop_alias indices => 'keys'

# prop_alias elems => 'values'

1;
# ABSTRACT: hash type

__END__

=pod

=encoding UTF-8

=head1 NAME

Data::Sah::Type::hash - hash type

=head1 VERSION

This document describes version 0.56 of Data::Sah::Type::hash (from Perl distribution Perinci-CmdLine-Any-Bundled version 0.02 version 0.01 version 0.01), released on 2015-05-13.

=for Pod::Coverage ^(clause_.+|clausemeta_.+)$

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Data-Sah>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Data-Sah>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Data-Sah>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
